'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});

var _three = require('three');

var THREE = _interopRequireWildcard(_three);

var _BasicVS = "varying vec2 vUv;\n\nvoid main() {\n    vUv = uv;\n    gl_Position = projectionMatrix * modelViewMatrix * vec4(position, 1.0);\n}\n";

var _BasicVS2 = _interopRequireDefault(_BasicVS);

var _EDLPassOneFS = "#include <packing>\nuniform sampler2D depthTexture;\nuniform sampler2D depthTextureHalfRes;\nuniform sampler2D depthTextureQuarterRes;\nuniform bool subsample;\nuniform vec2 resolution;\nuniform float strength;\nuniform float cameraNear;\nuniform float cameraFar;\n\nuniform int n;\nuniform int directions;\nuniform float radius;\n\nvarying vec2 vUv;\n\n#if 1\nfloat readDepth (sampler2D depthSampler, vec2 coord) {\n    float fragCoordZ = texture2D(depthSampler, coord).x;\n    float viewZ = perspectiveDepthToViewZ(fragCoordZ, cameraNear, cameraFar);\n    return log2(viewZToOrthographicDepth(viewZ, cameraNear, cameraFar));\n}\n#else\nfloat readDepth(sampler2D depthSampler, vec2 coord) {\n    return texture2D(depthSampler, coord).x;\n}\n#endif\n\n\n// inspiration from https://tel.archives-ouvertes.fr/tel-00438464/document and Potree\nvoid main() {\n    float zp = readDepth(depthTexture, vUv);\n    float zp1, zp2;\n    if (subsample) {\n        zp1 = readDepth(depthTextureHalfRes, vUv);\n        zp2 = readDepth(depthTextureQuarterRes, vUv);\n    }\n    float s = 0.0;\n\n    const int max_k = 16;\n    const int max_n = 4;\n\n    float step = 2.0 * 3.1415926 / float(directions);\n    for (int i=0; i<max_k; i++) {\n        if (i == directions) {\n            // workaround for loop index cannot be compared with non-constant expression\n            break;\n        }\n        for (int j=1; j<=max_n; j++) {\n            if (j > n) {\n                // workaround for loop index cannot be compared with non-constant expression\n                break;\n            }\n            float distance = radius * float(j);\n            float rad = float(i) * step;\n            vec2 offset = vec2(\n                cos(rad) * distance,\n                sin(rad) * distance) / resolution;\n\n            float zq = readDepth(depthTexture, vUv + offset);\n\n            s += max(0.0, -(zq - zp) / distance);\n\n            if (subsample) {\n                offset = 2.0 * offset;\n                float zq1 = readDepth(depthTextureHalfRes, vUv + offset);\n                s += max(0.0, -(zq1 - zp1) / distance);\n\n                offset = 2.0 * offset;\n                float zq2 = readDepth(depthTextureQuarterRes, vUv + offset);\n                s += max(0.0, -(zq2 - zp2) / distance);\n            }\n        }\n    }\n    s = s / float(directions) / float(n);\n    if (subsample) {\n        s = s / (float(directions) * float(n) * 2.0);\n    }\n\n    float A = 300.0 * strength;\n    s = exp(-s * A);\n    gl_FragColor = vec4(s, s, s, 1.0);\n}\n";

var _EDLPassOneFS2 = _interopRequireDefault(_EDLPassOneFS);

var _EDLPassTwoFS = "uniform sampler2D textureColor;\nuniform sampler2D textureEDL;\n\nvarying vec2 vUv;\n\nvoid main() {\n    float edl = texture2D(textureEDL, vUv).r;\n    // edl is 0 => no neighbours, so disable EDL to avoid drawing a black\n    // circle around individual points\n    if (edl == 0.0) {\n        edl = 1.0;\n    }\n    gl_FragColor = vec4(texture2D(textureColor, vUv).rgb * edl, 1.0);\n}\n";

var _EDLPassTwoFS2 = _interopRequireDefault(_EDLPassTwoFS);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj.default = obj; return newObj; } }

var RT = {
    COLOR_AND_DEPTH: 0,
    EDL_VALUES: 1,
    DEPTH_BUFFER_HALF_RES: 2,
    DEPTH_BUFFER_QUARTER_RES: 3
};

function EDLRenderer(view) {
    var _this = this;

    this.edlScene = new THREE.Scene();
    this.quad = new THREE.Mesh(new THREE.PlaneBufferGeometry(2, 2), null);
    this.quad.frustumCulled = false;
    this.edlScene.add(this.quad);
    this.edlCam = new THREE.OrthographicCamera(-1, 1, 1, -1, 0, 10);

    this.edlMaterial = new THREE.ShaderMaterial({
        uniforms: {
            depthTexture: { value: null },
            depthTextureHalfRes: { value: null },
            depthTextureQuarterRes: { value: null },
            subsample: { value: false },
            resolution: { value: new THREE.Vector2(256, 256) },
            cameraNear: { value: 0.01 },
            cameraFar: { value: 100 },
            radius: { value: 0 },
            strength: { value: 0 },
            directions: { value: 0 },
            n: { value: 0 }

        },
        vertexShader: _BasicVS2.default,
        fragmentShader: _EDLPassOneFS2.default
    });

    this.composeMaterial = new THREE.ShaderMaterial({
        uniforms: {
            textureColor: { value: null },
            textureEDL: { value: null }
        },
        vertexShader: _BasicVS2.default,
        fragmentShader: _EDLPassTwoFS2.default
    });

    this.renderTargets = [null, null, null, null];

    this.controlParameters = {
        subsample: false,
        radius: 3.0,
        strength: 1.0,
        directions: 8,
        n: 1
    };

    this._createRenderTargets(view);

    view.addEventListener('resize', function () {
        _this.renderTargets.forEach(function (rt) {
            return rt.dispose();
        });
        _this._createRenderTargets(view);
    });
}

EDLRenderer.prototype._createRenderTargets = function _createRenderTargets(view) {
    this.renderTargets[RT.COLOR_AND_DEPTH] = new THREE.WebGLRenderTarget(view.camera.width, view.camera.height);
    this.renderTargets[RT.EDL_VALUES] = new THREE.WebGLRenderTarget(view.camera.width, view.camera.height);
    this.renderTargets[RT.DEPTH_BUFFER_HALF_RES] = new THREE.WebGLRenderTarget(view.camera.width * 0.5, view.camera.height * 0.5);
    this.renderTargets[RT.DEPTH_BUFFER_QUARTER_RES] = new THREE.WebGLRenderTarget(view.camera.width * 0.25, view.camera.height * 0.25);

    this.renderTargets[RT.COLOR_AND_DEPTH].texture.minFilter = THREE.LinearFilter;
    this.renderTargets[RT.COLOR_AND_DEPTH].texture.generateMipmaps = false;
    this.renderTargets[RT.COLOR_AND_DEPTH].depthBuffer = true;
    this.renderTargets[RT.COLOR_AND_DEPTH].texture.format = THREE.RGBAFormat;
    this.renderTargets[RT.COLOR_AND_DEPTH].texture.minFilter = THREE.NearestFilter;
    this.renderTargets[RT.COLOR_AND_DEPTH].texture.magFilter = THREE.NearestFilter;
    this.renderTargets[RT.COLOR_AND_DEPTH].depthTexture = new THREE.DepthTexture();
    this.renderTargets[RT.COLOR_AND_DEPTH].depthTexture.type = THREE.UnsignedShortType;

    this.renderTargets[RT.EDL_VALUES] = new THREE.WebGLRenderTarget(view.camera.width, view.camera.height);
    this.renderTargets[RT.EDL_VALUES].texture.generateMipmaps = false;
    this.renderTargets[RT.EDL_VALUES].depthBuffer = false;
    this.renderTargets[RT.EDL_VALUES].texture.format = THREE.RGBAFormat;
    this.renderTargets[RT.EDL_VALUES].texture.minFilter = THREE.NearestFilter;
    this.renderTargets[RT.EDL_VALUES].texture.magFilter = THREE.NearestFilter;

    this.renderTargets[RT.DEPTH_BUFFER_HALF_RES] = new THREE.WebGLRenderTarget(view.camera.width * 0.5, view.camera.height * 0.5);
    this.renderTargets[RT.DEPTH_BUFFER_HALF_RES].texture.generateMipmaps = false;
    this.renderTargets[RT.DEPTH_BUFFER_HALF_RES].depthBuffer = false;
    this.renderTargets[RT.DEPTH_BUFFER_HALF_RES].texture.format = THREE.RGBAFormat;
    this.renderTargets[RT.DEPTH_BUFFER_HALF_RES].texture.minFilter = THREE.NearestFilter;
    this.renderTargets[RT.DEPTH_BUFFER_HALF_RES].texture.magFilter = THREE.NearestFilter;

    this.renderTargets[RT.DEPTH_BUFFER_QUARTER_RES] = new THREE.WebGLRenderTarget(view.camera.width * 0.25, view.camera.height * 0.25);
    this.renderTargets[RT.DEPTH_BUFFER_QUARTER_RES].texture.generateMipmaps = false;
    this.renderTargets[RT.DEPTH_BUFFER_QUARTER_RES].depthBuffer = false;
    this.renderTargets[RT.DEPTH_BUFFER_QUARTER_RES].texture.format = THREE.RGBAFormat;
    this.renderTargets[RT.DEPTH_BUFFER_QUARTER_RES].texture.minFilter = THREE.NearestFilter;
    this.renderTargets[RT.DEPTH_BUFFER_QUARTER_RES].texture.magFilter = THREE.NearestFilter;
};

EDLRenderer.prototype.renderView = function renderView(view) {
    var g = view.mainLoop.gfxEngine;
    var r = g.renderer;

    // render to target
    r.setRenderTarget(this.renderTargets[RT.COLOR_AND_DEPTH]);
    r.clear();
    r.setViewport(0, 0, this.renderTargets[RT.COLOR_AND_DEPTH].width, this.renderTargets[RT.COLOR_AND_DEPTH].height);
    r.render(view.scene, view.camera.camera3D, this.renderTargets[RT.COLOR_AND_DEPTH]);

    if (this.controlParameters.subsample) {
        // sub-sample depth buffer
        this.quad.material = new THREE.MeshBasicMaterial({ map: this.renderTargets[RT.COLOR_AND_DEPTH].depthTexture });
        r.setRenderTarget(this.renderTargets[RT.DEPTH_BUFFER_HALF_RES]);
        r.clear();
        r.setViewport(0, 0, this.renderTargets[RT.DEPTH_BUFFER_HALF_RES].width, this.renderTargets[RT.DEPTH_BUFFER_HALF_RES].height);
        r.render(this.edlScene, this.edlCam, this.renderTargets[RT.DEPTH_BUFFER_HALF_RES]);

        r.setRenderTarget(this.renderTargets[RT.DEPTH_BUFFER_QUARTER_RES]);
        r.clear();
        r.setViewport(0, 0, this.renderTargets[RT.DEPTH_BUFFER_QUARTER_RES].width, this.renderTargets[RT.DEPTH_BUFFER_QUARTER_RES].height);
        r.render(this.edlScene, this.edlCam, this.renderTargets[RT.DEPTH_BUFFER_QUARTER_RES]);
    }

    // draw edl
    this.quad.material = this.edlMaterial;
    this.quad.material.uniforms.depthTexture.value = this.renderTargets[RT.COLOR_AND_DEPTH].depthTexture;
    this.quad.material.uniforms.depthTextureHalfRes.value = this.renderTargets[RT.DEPTH_BUFFER_HALF_RES].texture;
    this.quad.material.uniforms.depthTextureQuarterRes.value = this.renderTargets[RT.DEPTH_BUFFER_QUARTER_RES].texture;
    this.quad.material.uniforms.resolution.value.set(this.renderTargets[RT.COLOR_AND_DEPTH].width, this.renderTargets[RT.COLOR_AND_DEPTH].height);
    this.quad.material.uniforms.cameraNear.value = view.camera.camera3D.near;
    this.quad.material.uniforms.cameraFar.value = view.camera.camera3D.far;
    this.quad.material.uniforms.radius.value = this.controlParameters.radius;
    this.quad.material.uniforms.strength.value = this.controlParameters.strength;
    this.quad.material.uniforms.directions.value = this.controlParameters.directions;
    this.quad.material.uniforms.n.value = this.controlParameters.n;
    this.quad.material.uniforms.subsample.value = this.controlParameters.subsample;

    r.setRenderTarget(this.renderTargets[RT.EDL_VALUES]);
    r.clear();
    r.setViewport(0, 0, this.renderTargets[RT.EDL_VALUES].width, this.renderTargets[RT.EDL_VALUES].height);
    r.render(this.edlScene, this.edlCam, this.renderTargets[RT.EDL_VALUES]);

    // combine ...
    this.quad.material = this.composeMaterial;
    this.quad.material.uniforms.textureColor.value = this.renderTargets[RT.COLOR_AND_DEPTH].texture;
    this.quad.material.uniforms.textureEDL.value = this.renderTargets[RT.EDL_VALUES].texture;
    r.setRenderTarget();
    r.clear();
    r.setViewport(0, 0, g.getWindowSize().x, g.getWindowSize().y);
    r.render(this.edlScene, this.edlCam);
};

exports.default = EDLRenderer;