'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});

var _getIterator2 = require('babel-runtime/core-js/get-iterator');

var _getIterator3 = _interopRequireDefault(_getIterator2);

var _assign = require('babel-runtime/core-js/object/assign');

var _assign2 = _interopRequireDefault(_assign);

var _set = require('babel-runtime/core-js/set');

var _set2 = _interopRequireDefault(_set);

var _three = require('three');

var THREE = _interopRequireWildcard(_three);

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj.default = obj; return newObj; } }

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var maxFOV = 90; // TODO
// - rotation de côté de la tête
// - clean
// - examples

var rotationFactor = 0.00012; // determined empirically

var MOVEMENTS = {
    38: { method: 'translateZ', sign: -1 }, // FORWARD: up key
    40: { method: 'translateZ', sign: 1 }, // BACKWARD: down key
    37: { method: 'translateX', sign: -1 }, // STRAFE_LEFT: left key
    39: { method: 'translateX', sign: 1 }, // STRAFE_RIGHT: right key
    33: { method: 'rotateZ', sign: 1, noSpeed: true }, // UP: PageUp key
    34: { method: 'rotateZ', sign: -1, noSpeed: true }, // DOWN: PageDown key
    wheelup: { method: 'translateZ', sign: 1, oneshot: true }, // WHEEL up
    wheeldown: { method: 'translateZ', sign: -1, oneshot: true } // WHEEL down
};

function FPControls(view) {
    var domElement = view.mainLoop.gfxEngine.renderer.domElement;
    this.view = view;
    this._camera3D = view.camera.camera3D;
    this.moves = new _set2.default();
    this.moveSpeed = 10; // backward or forward move speed in m/s

    this._onMouseDownMouseX = 0;
    this._onMouseDownMouseY = 0;
    this._onMouseDownTheta = 0;
    this._onMouseDownPhi = 0;

    this._isUserInteracting = false;

    domElement.addEventListener('mousedown', onDocumentMouseDown.bind(this), false);
    domElement.addEventListener('touchstart', onTouchStart.bind(this), false);
    domElement.addEventListener('mousemove', onDocumentMouseMove.bind(this), false);
    domElement.addEventListener('touchmove', onTouchMove.bind(this), false);
    domElement.addEventListener('mouseup', onDocumentMouseUp.bind(this), false);
    domElement.addEventListener('touchend', onDocumentMouseUp.bind(this), false);
    domElement.addEventListener('mousewheel', onDocumentMouseWheel.bind(this), false);
    domElement.addEventListener('DOMMouseScroll', onDocumentMouseWheel.bind(this), false); // firefox
    domElement.addEventListener('keyup', onKeyUp.bind(this), true);
    domElement.addEventListener('keydown', onKeyDown.bind(this), true);

    this.view.addFrameRequester(this);
}

(0, _assign2.default)(FPControls.prototype, THREE.EventDispatcher.prototype);
FPControls.prototype.constructor = FPControls;

function onDocumentMouseDown(event) {
    event.preventDefault();
    this._isUserInteracting = true;

    this._onMouseDownMouseX = event.clientX;
    this._onMouseDownMouseY = event.clientY;
    this._onMouseDownTheta = this._camera3D.rotation.z;
    this._onMouseDownPhi = this._camera3D.rotation.x;
}

function onTouchStart(event) {
    event.preventDefault();
    this._isUserInteracting = true;

    this._onMouseDownMouseX = event.touches[0].pageX;
    this._onMouseDownMouseY = event.touches[0].pageY;
    this._onMouseDownTheta = this._camera3D.rotation.z;
    this._onMouseDownPhi = this._camera3D.rotation.x;
}

function onTouchMove(event) {
    if (this._isUserInteracting === true) {
        var fovCorrection = this._camera3D.fov / maxFOV; // 1 at maxFOV
        this._camera3D.rotateY((this._onMouseDownMouseX - event.touches[0].pageX) * rotationFactor * fovCorrection);
        this._camera3D.rotateX((event.touches[0].pageY - this._onMouseDownMouseY) * rotationFactor * fovCorrection);
        this.view.notifyChange(false);
    }
}

function onDocumentMouseMove(event) {
    if (this._isUserInteracting === true) {
        var fovCorrection = this._camera3D.fov / maxFOV; // 1 at maxFOV
        this._camera3D.rotateY((event.clientX - this._onMouseDownMouseX) * rotationFactor * fovCorrection);
        this._camera3D.rotateX((event.clientY - this._onMouseDownMouseY) * rotationFactor * fovCorrection);
        this._onMouseDownMouseX = event.clientX;
        this._onMouseDownMouseY = event.clientY;
        this.view.notifyChange(false);
    }
}

function onDocumentMouseUp() {
    this._isUserInteracting = false;
}

function onKeyUp(e) {
    var move = MOVEMENTS[e.keyCode];
    if (move) {
        this.moves.delete(move);
        this.view.notifyChange(true);
        e.preventDefault();
    }
}

function onKeyDown(e) {
    var move = MOVEMENTS[e.keyCode];
    if (move) {
        this.moves.add(move);
        this.view.notifyChange(false);
        e.preventDefault();
    }
}

function onDocumentMouseWheel(event) {
    var delta = 0;
    if (event.wheelDelta !== undefined) {
        delta = event.wheelDelta;
        // Firefox
    } else if (event.detail !== undefined) {
        delta = -event.detail;
    }
    if (delta < 0) {
        this.moves.add(MOVEMENTS.wheelup);
    } else {
        this.moves.add(MOVEMENTS.wheeldown);
    }

    this.view.notifyChange(false);
}

FPControls.prototype.isUserInteracting = function isUserInteracting() {
    return this.moves.size !== 0;
};

FPControls.prototype.update = function update(dt) {
    // if we are in a keypressed state, then update position

    // dt is 0 when we just started rendering, we consider a 1-frame move in this case
    if (dt === 0) {
        dt = 0.016;
    }

    var _iteratorNormalCompletion = true;
    var _didIteratorError = false;
    var _iteratorError = undefined;

    try {
        for (var _iterator = (0, _getIterator3.default)(this.moves), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
            var _move = _step.value;

            this._camera3D[_move.method](_move.sign * (_move.noSpeed ? 1 : this.moveSpeed) * dt);
        }
    } catch (err) {
        _didIteratorError = true;
        _iteratorError = err;
    } finally {
        try {
            if (!_iteratorNormalCompletion && _iterator.return) {
                _iterator.return();
            }
        } finally {
            if (_didIteratorError) {
                throw _iteratorError;
            }
        }
    }

    if (this.moves.size > 0 || this._isUserInteracting) {
        this._camera3D.updateMatrix();
        this._camera3D.updateMatrixWorld();
        this.view.notifyChange(true);

        var _iteratorNormalCompletion2 = true;
        var _didIteratorError2 = false;
        var _iteratorError2 = undefined;

        try {
            for (var _iterator2 = (0, _getIterator3.default)(this.moves), _step2; !(_iteratorNormalCompletion2 = (_step2 = _iterator2.next()).done); _iteratorNormalCompletion2 = true) {
                var move = _step2.value;

                if (move.oneshot) {
                    this.moves.delete(move);
                }
            }
        } catch (err) {
            _didIteratorError2 = true;
            _iteratorError2 = err;
        } finally {
            try {
                if (!_iteratorNormalCompletion2 && _iterator2.return) {
                    _iterator2.return();
                }
            } finally {
                if (_didIteratorError2) {
                    throw _iteratorError2;
                }
            }
        }
    }
};

exports.default = FPControls;