'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});

var _getIterator2 = require('babel-runtime/core-js/get-iterator');

var _getIterator3 = _interopRequireDefault(_getIterator2);

var _three = require('three');

var THREE = _interopRequireWildcard(_three);

var _earcut = require('earcut');

var _earcut2 = _interopRequireDefault(_earcut);

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj.default = obj; return newObj; } }

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function coordinatesToMesh(coordinates) {
    if (!coordinates) {
        return;
    }
    // create geometry
    var geometry = new THREE.BufferGeometry();

    var vertices = new Float32Array(3 * coordinates.coordinates.length);
    var offset = 0;
    var _iteratorNormalCompletion = true;
    var _didIteratorError = false;
    var _iteratorError = undefined;

    try {
        for (var _iterator = (0, _getIterator3.default)(coordinates.coordinates), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
            var coordinate = _step.value;

            vertices[offset] = coordinate._values[0];
            vertices[offset + 1] = coordinate._values[1];
            vertices[offset + 2] = coordinate._values[2];
            offset += 3;
        }
    } catch (err) {
        _didIteratorError = true;
        _iteratorError = err;
    } finally {
        try {
            if (!_iteratorNormalCompletion && _iterator.return) {
                _iterator.return();
            }
        } finally {
            if (_didIteratorError) {
                throw _iteratorError;
            }
        }
    }

    geometry.addAttribute('position', new THREE.BufferAttribute(vertices, 3));

    // build indice and instanciate mesh
    var result = void 0;
    switch (coordinates.type) {
        case 'point':
            result = new THREE.Points(geometry);
            break;
        case 'linestring':
            {
                var indices = [];
                /* eslint-disable guard-for-in */
                for (var id in coordinates.featureVertices) {
                    var line = coordinates.featureVertices[id];
                    // TODO optimize indices lines
                    // is the same array each time
                    for (var i = line.offset; i < line.offset + line.count - 1; ++i) {
                        indices.push(i);
                        indices.push(i + 1);
                    }
                }
                geometry.setIndex(new THREE.BufferAttribute(new Uint16Array(indices), 1));
                result = new THREE.LineSegments(geometry);
                break;
            }
        case 'polygon':
            {
                var _indices = [];
                /* eslint-disable guard-for-in */
                for (var _id in coordinates.featureVertices) {
                    var polygon = coordinates.featureVertices[_id];
                    var contour = vertices.slice(polygon.offset * 3, (polygon.offset + polygon.count) * 3);
                    var triangles = (0, _earcut2.default)(contour, null, 3);
                    var _iteratorNormalCompletion2 = true;
                    var _didIteratorError2 = false;
                    var _iteratorError2 = undefined;

                    try {
                        for (var _iterator2 = (0, _getIterator3.default)(triangles), _step2; !(_iteratorNormalCompletion2 = (_step2 = _iterator2.next()).done); _iteratorNormalCompletion2 = true) {
                            var indice = _step2.value;

                            _indices.push(polygon.offset + indice);
                        }
                    } catch (err) {
                        _didIteratorError2 = true;
                        _iteratorError2 = err;
                    } finally {
                        try {
                            if (!_iteratorNormalCompletion2 && _iterator2.return) {
                                _iterator2.return();
                            }
                        } finally {
                            if (_didIteratorError2) {
                                throw _iteratorError2;
                            }
                        }
                    }
                }
                geometry.setIndex(new THREE.BufferAttribute(new Uint16Array(_indices), 1));
                result = new THREE.Mesh(geometry);
                break;
            }
        default:
    }

    result.featureVertices = coordinates.featureVertices;

    return result;
}

function featureToThree(feature) {
    var mesh = coordinatesToMesh(feature.geometry);
    mesh.properties = feature.properties;
    return mesh;
}

function featureCollectionToThree(featureCollection) {
    var group = new THREE.Group();
    var _iteratorNormalCompletion3 = true;
    var _didIteratorError3 = false;
    var _iteratorError3 = undefined;

    try {
        for (var _iterator3 = (0, _getIterator3.default)(featureCollection.geometries), _step3; !(_iteratorNormalCompletion3 = (_step3 = _iterator3.next()).done); _iteratorNormalCompletion3 = true) {
            var geometry = _step3.value;

            group.add(coordinatesToMesh(geometry));
        }
    } catch (err) {
        _didIteratorError3 = true;
        _iteratorError3 = err;
    } finally {
        try {
            if (!_iteratorNormalCompletion3 && _iterator3.return) {
                _iterator3.return();
            }
        } finally {
            if (_didIteratorError3) {
                throw _iteratorError3;
            }
        }
    }

    group.features = featureCollection.features;
    return group;
}

exports.default = {
    convert: function convert(feature) {
        if (feature.geometries) {
            return featureCollectionToThree(feature);
        } else {
            return featureToThree(feature);
        }
    }
};